r"""
A module that holds setup related data.
"""

Defaults = {

	'projectName': '<unnamed>',

	'delay': 2.0, # The standard wait time.

	'devMode': False,

	'logLevel': 2,

	'fsRoot': '.' # # Risky filesystem operations such as removePath are limited to fsRoot, which defaults to the CWD. For a tighter control, set this var to an absolute path.
}



r"""
dev
---

A module to help with develpment.

#ToDo: Ensure that unicode's are handled well by get pretty.
#Later: Think of adding some functions, like scan, to the built-ins, so that there can be used without importing.
"""
from laufire.flow import forgive
from laufire.extensions import pairs, isIterable

# Workers
def _getPretty(Iterable, indent):
	ret = ''

	for key, value in pairs(Iterable):
		if isIterable(value):
			ret += u'%s%s:\n%s\n' % (u'\t' * indent, unicode(key), _getPretty(value, indent + 1))

		else:
			ret += u'{0}{1}: {2}\n'.format(u'\t' * indent, unicode(key), unicode(value))

	return ret.encode('utf-8')

def interactive(func, message=None, raiseError=False):
	r"""Helps with re-running tasks till there were no errors.
	"""
	while True:
		e = forgive(func)

		if not e:
			return

		print e

		if raw_input((message or 'Fix and continue...') + ' (Y/n):').lower() == 'n':
			if raiseError:
				raise e

			return e

def pause(message='Paused! Press return to continue...'):
	raw_input(message)

def peek(val):
	print val
	return val

def hl(val, color='LIGHTYELLOW_EX'): # highlight
	from laufire.logger import log

	log(val, color)

	return val

def tee(val, func):
	r"""Tee-s given value into the given function.
	"""
	func(val)
	return val

def details(Obj):
	for attr in [attr for attr in dir(Obj) if attr[0] != '_']:
		print '%s: %s' % (attr, getattr(Obj, attr))

	return Obj

# Makes pretty the given iterable (dictionary, list etc).
def getPretty(Iterable):
	if not isIterable(Iterable):
		return str(Iterable)

	return _getPretty(Iterable, 0).replace('\n\n\n', '\n\n')

# Pretty prints the given iterable.
def pPrint(obj):
	print getPretty(obj) if isIterable(obj) else obj
	return obj

# Plots the given list of numbers.
def plot(Values):
	import matplotlib.pyplot as plt

	plt.plot(Values)
	plt.grid(True)

	plt.show()



r"""A module to help reading and writing TSV (tab separated files).

#Note: TSV is chosen over CSV for their simplicity.
"""
import csv
from os.path import exists

class TSV:
	def __init__(self, filePath=None, mode='dict', createMissingFile=False): # #Note: Mode could be set to 'list' to handle data as list of lists.
		r"""
		A simple class to help with reading and writing TSV files.
		"""
		self._file = None

		if filePath:
			if createMissingFile and not exists(filePath):
				file = open(filePath, 'wb+')
				file.close()

			self.open(filePath, mode)

	def open(self, filePath, mode='dict'):
		r"""Opens the given file.
		"""
		if self._file:
			self.close()

		self._filePath = filePath
		self._file = open(filePath, 'rb+')

		if mode == 'dict':
			self._reader = csv.DictReader(self._file, delimiter='\t', lineterminator='\n')
			self.FieldNames = self._reader.fieldnames
			self._writer = csv.DictWriter(self._file, self.FieldNames, delimiter='\t', lineterminator='\n')

		else:
			self._reader = csv.reader(self._file, delimiter='\t', lineterminator='\n')
			self.FieldNames = None
			self._writer = csv.writer(self._file, delimiter='\t', lineterminator='\n')

	def read(self):
		r"""Returns an iterable reader.
		"""
		return self._reader

	def write(self, *Rows): #Pending: Make the call to utilize iterators.
		r"""Appends the given rows to the file.
		"""
		return self._writer.writerows(Rows)

	def empty(self):
		r"""Clears all the data from the file.
		"""
		self._file.seek(0)
		self._file.truncate()

		if self.FieldNames:
			self._writer.writeheader()

	def close(self):
		r"""Closes the file.
		"""
		self._file.close()
		self._file = None

def readTSV(filePath, mode='dict'):
	r"""Reads and returns every row from the given TSV file.

	Args:
		filePath (str): The path to the TSV file.
		mode (dict/list): Sets the type of the items in the returned list.
	"""
	TsvFile = TSV(filePath, mode)

	Rows = [row for row in TsvFile.read()]

	TsvFile.close()
	return Rows



r"""
parse
=====

A set of functions to parse data files of various data formats.
"""
from os.path import splitext

# Helpers
def getParser(filePath):
	return ParserMap[splitext(filePath)[1][1:].lower()]

# Exports
def parseYAML(filePath):
	from laufire.yamlex import YamlEx

	Data = YamlEx(filePath)
	Data.interpolate()

	return Data

def parseJSON(filePath):
	import json

	with open(filePath, 'r') as file:
		return json.load(file)

def parse(filePath, format=None):
	r"""Parses data from the diven file.

	Args:
		filePath (str): The path to process.
		format (str): The format of the file. When not given, it is detected based on the files extension. For supported formats check the var ParserMap.
	"""
	return (ParserMap[format] if format else getParser(filePath))(filePath)

# Data
ParserMap = {'yml': parseYAML, 'yaml': parseYAML, 'json': parseJSON}



r"""
A module to help with changing envirnment settings.
"""
def enableUnicodePrinting():
	r"""
	Enables the printing of unicode characters, bot through stdout and stderr.
	"""
	import codecs
	import sys

	sys.stdout = codecs.getwriter('utf8')(sys.stdout)
	sys.stderr = codecs.getwriter('utf8')(sys.stderr)



import sqlite3

# #Pending: Raise exceptions on constraint failures etc.

# Helpers
def dictFactory(cursor, row):
	d = {}
	for idx, col in enumerate(cursor.description):
		d[col[0]] = row[idx]

	return d

# Exports
def execute(filePath, query, useDict=False):
	r"""Executes the given query over the given DB and returns the results, if any.
	"""
	DB = SQLiteDB(filePath, useDict)
	return DB.execute(query).fetchall()

class SQLiteDB:
	def __init__(self, filePath=':memory:', useDict=False):
		r"""
		A simple wrapper over native sqlite3, to ease development.

		Args:
			filePath (str): The path to the sqlite file.
			useDict (bool, Fale): SELECT-s return a list of dictionaries when set to True.

		Note:
			The calss handles a single DB and allows only a single cursor over a single connection.
		"""
		self._inited = False # Used to break getattr chains, that occur on connection opening errors.

		self.path = filePath
		self._conn = sqlite3.connect(self.path)

		if useDict:
			self._conn.row_factory = dictFactory

		self._cur = self._conn.cursor()

		self._inited = True

	def __del__(self):
		if self._inited:
			self.close()

	def execFile(self, filePath):
		r"""Helps with executing sql files.
		"""
		for line in open(filePath):
			self.execute(line.rstrip('\r\n '))

	def close(self):
		r"""Close the DB.

		Any uncommited changes will be commited before closing.
		"""
		if self._conn:
			self._conn.commit()
			self._conn.close()
			self._conn = None

	def reopen(self):
		r"""Reopens a closed DB.
		"""
		self.__init__(self.path)

	def __getattr__(self, attr):
		r"""
		Allows the access of the methods of the underlying cursor / connection.
		"""
		return getattr(self._cur, attr) if hasattr(self._cur, attr) else getattr(self._conn, attr)

class SQLiteSimpleTable(SQLiteDB):
	def __init__(self, filePath=':memory:', tableName='unnamed', key=None):
		r"""
		An extension to the class, SQLiteDB that simplifies the access of tables with a primary key.

		The in and out values are simple dicts, with keys matching the field names of the table.

		#Note: When no key is specified the key is got from the table info.
		"""
		SQLiteDB.__init__(self, filePath, True)
		self.tableName = tableName

		self.execute("PRAGMA table_info(`%s`);" % tableName)

		Record = self.fetchone()

		if not Record:
			raise Exception('No such table %s, in the DB: %s' % (tableName, filePath))

		if not key:
			while Record:
				if Record['pk'] == 1:
					key = Record['name']
					break

				Record = self.fetchone()

		if not key:
			raise Exception('No primary key found for the table %s, in the DB: %s' % (tableName, filePath))

		self._key = key
		self._Statements = {

			'get': "SELECT * FROM `%s` WHERE `%s`=?;" % (tableName, key),
			'getColTpl': "SELECT %s, `%%s` FROM `%s`;" % (key, tableName),
			'getAll': "SELECT * FROM `%s`;" % tableName,
			'del': "DELETE FROM `%s` WHERE `%s`=?;" % (tableName, key),
			'update': "UPDATE OR IGNORE %s SET %%s WHERE `%s`=:%s;" % (tableName, key, key),
			'insert': "INSERT OR IGNORE INTO %s (%%s) VALUES (%%s);" % tableName,
		}

	def get(self, key):
		self.execute(self._Statements['get'], [key])
		return self.fetchone() or {}

	def getCol(self, colName):
		Ret = {}
		keyCol = self._key

		for Item in self.execute(self._Statements['getColTpl'] % colName).fetchall():
			Ret[Item[keyCol]] = Item[colName]

		return Ret

	def getAll(self):
		Ret = {}
		keyCol = self._key

		for Item in self.execute(self._Statements['getAll']).fetchall():
			Ret[Item[keyCol]] = Item

		return Ret

	def set(self, Values, dontInsert=False):
		# #From: http://stackoverflow.com/questions/14108162/python-sqlite3-insert-into-table-valuedictionary-goes-here
		Keys = [key for key in Values.keys() if key != self._key]
		updateStr = ','.join(['`%s`=:%s' % (key, key) for key in Keys])

		Keys = Values.keys()
		columns = '`%s`' % '`,`'.join(Keys)
		placeholders = ':'+',:'.join(Keys)

		self.execute(self._Statements['update'] % updateStr, Values)

		if not dontInsert:
			self.execute(self._Statements['insert'] % (columns, placeholders), Values)

		self.commit()

	def delete(self, key):
		self.execute(self._Statements['del'], [key])
		self.commit()

	def reopen(self):
		self.__init__(self.path, self.tableName, self._key)

def importTables(toDBPath, fromDBPath, TableNames):
	TargetDB = SQLiteDB(toDBPath)
	TargetDB.execute('ATTACH ? as patch', [fromDBPath])

	for tableName in TableNames:
		TargetDB.execute('INSERT OR REPLACE INTO {0} SELECT * FROM patch.{0};'.format(tableName))

	TargetDB.execute('DETACH patch;')
	TargetDB.execute("VACUUM;")
	TargetDB.commit()
	TargetDB.close()

def importTablesFromFile(toDBPath, fromFilePath, tableName=None, delimiter='\t', lineterminator='\n'):
	import csv

	if not tableName:
		from os.path import splitext, basename
		tableName = basename(splitext(fromFilePath)[0]) # Use the file name as the table name.

	Rows = [Row for Row in csv.reader(open(fromFilePath, 'rb+'), delimiter=delimiter, lineterminator=lineterminator)]

	if not Rows:
		return

	TargetDB = SQLiteDB(toDBPath)
	TargetDB.executemany('INSERT OR REPLACE INTO {0} VALUES({1});'.format(tableName, ','.join(['?'] * len(Rows[0]))), Rows)
	TargetDB.execute("VACUUM;")
	TargetDB.commit()
	TargetDB.close()



r"""
OS Bridge
=========

A multi platform adapter.
"""
import os
from os.path import expanduser
import platform

# Data
platform = platform.system()

def getOSRoot():
	return '' if platform != 'Windows' else os.environ['windir'][:2] # #Note: An empty string is returned, as it's the projets standard to always join the paths with a separator.

def getDataDir():
	userPath = expanduser('~')
	return userPath if  platform != 'Windows' else ('%s\\AppData\\Roaming' % userPath)



r"""
A module to help with making the module, filesys, to wotk across platforms.

# Later: Test the exports on multiple platforms.
"""
import platform

# Data
system = platform.system()

if system != 'Windows':
	from os import unlink
	from os.path import isdir, islink

	# Exports
	# #Note: This calls won't be available on Windows.
	from os import link, symlink #pylint: disable=W0611

	rmlink = unlink

	def isLinkedDir(targetPath):
		return isdir(targetPath) and islink(targetPath)

else:
	from os.path import abspath, isdir
	from win32com.shell import shell
	from win32file import FILE_ATTRIBUTE_DIRECTORY, GetFileAttributes
	from win32file import CreateHardLink, CreateSymbolicLink, RemoveDirectory as rmlink #pylint: disable=W0611

	# Constants
	FILE_ATTRIBUTE_REPARSE_POINT = 1024
	REPARSE_FOLDER = (FILE_ATTRIBUTE_DIRECTORY | FILE_ATTRIBUTE_REPARSE_POINT)

	# Exports
	def link(sourcePath, targetPath):
		CreateHardLink(targetPath, sourcePath)

	def symlink(sourcePath, targetPath):
		r"""
		Creates a soft link for the given paths.
		"""
		CreateSymbolicLink(targetPath, sourcePath, isdir(sourcePath))

	def isLinkedDir(targetPath):
		r"""Detects whether the given path is a NTFS junction.

		#Later: Fix: This call fails when the target points to an invalid location.
		"""
		# #From: http://stackoverflow.com/questions/1447575/symlinks-on-windows
		result = GetFileAttributes(targetPath)
		return result > -1 and result & REPARSE_FOLDER == REPARSE_FOLDER

	# #Pending: Create a standard API for the following functions, across platforms.
	import pythoncom
	import win32con
	from win32api import SetFileAttributes

	def setAttrs(target, *Attrs):
		r"""Sets the given attributes to the given target.

		Args:
			target  (path): The target path to set the attributes.
			*Attrs : One or more attribute names of the `SetFileAttributes <https://msdn.microsoft.com/en-us/library/windows/desktop/aa365535(v=vs.85).aspx>`_ function without the preceeding *FILE_ATTRIBUTE_*.
		"""
		attr = 0
		for item in Attrs:
			attr = attr | getattr(win32con, 'FILE_ATTRIBUTE_%s' % item.upper())

		SetFileAttributes(target, attr)

	def createShortcut(toDocPath, fromLinkPath):
		shortcut = pythoncom.CoCreateInstance(
			shell.CLSID_ShellLink,
			None,
			pythoncom.CLSCTX_INPROC_SERVER, #pylint: disable=E1101
			shell.IID_IShellLink
			)

		shortcut.SetPath(abspath(toDocPath))

		persistFile = shortcut.QueryInterface(pythoncom.IID_IPersistFile) #pylint: disable=E1101
		persistFile.Save(abspath(fromLinkPath), 0)



r"""A module to help with shell calls.

# #Note: The processes with huge amounts of stdout or stderr could hang.
"""
import sys
import os

from subprocess import Popen, PIPE
from shlex import split as shlexSplit

from laufire.dev import getPretty
from laufire.logger import debug, dump

# State
split = None

# Helpers
def getNthLine(string, N):
	return string.strip().split('\n')[N]

# Exports
def run(command, **KWArgs):
	r"""Starts a process, waits till the process completes and returns the return-code.

	#Tip: Use this method to live stream output from the command.
	"""
	debug(command)
	dump(getPretty(KWArgs))

	p = Popen(split(command), **KWArgs)
	p.wait()

	return p.returncode

def call(command, **KWArgs): # from gitapi.py
	r"""Starts a process, waits till the process completes and returns a dictionary with the return-code, stdout and stderr.

	#Tip: Use this method when there's a need to process stdout or stderr.
	"""
	debug(command)
	dump(getPretty(KWArgs))

	p = Popen(split(command), stdout=PIPE, stderr=PIPE, **KWArgs)
	out, err = [x.decode('utf-8') for x in p.communicate()]

	return {'out': out, 'err': err, 'code': p.returncode}

def piped(*Commands, **KWArgs):
	r"""Emulates piped commands in *nix systems. Returns a dictionary with the final return-code, stdout and a stderr.
	"""
	dump(getPretty(KWArgs))

	out = None
	err = None
	code = 0

	for command in Commands:
		debug(command)

		p = Popen(split(command), stdout=PIPE, stderr=PIPE, stdin=PIPE, **KWArgs)
		out, err = p.communicate(out)
		code = p.returncode

		if code:
			break

	return {'out': out, 'err': err, 'code': code}

def writable(command, data, **KWArgs):
	r"""Opens a process and writes the given data to its STDIN.

	The newline character could be used to separate multiple lines.
	"""
	debug(command)
	dump(getPretty(KWArgs))
	# #Note: data isn't dumped, to keep it secure.

	p = Popen(split(command), stdout=PIPE, stderr=PIPE, stdin=PIPE, **KWArgs)
	out, err = p.communicate(data)

	code = p.returncode

	return {'out': out, 'err': err, 'code': code}

def debugCall(command, **KWArgs):
	r"""Starts a process, waits till the process completes and returns a dictionary with the return-code, stdout and stderr.

	#Tip: Use this method call scripts during development, errors would be logged to the live stderr, at the same time stdout could be buffered for processing.
	#Tip: A modified pdb like, modPdb = pdb.Pdb(stdout=sys.__stderr__), could be used to debug scripts in stderr.
	"""
	debug(command)
	dump(getPretty(KWArgs))

	p = Popen(split(command), stdout=PIPE, **KWArgs)

	return getProcessData(p)

def launch(command, **KWArgs):
	r"""Launches a process and quits without waiting for its completion.
	"""
	debug(command)
	dump(getPretty(KWArgs))

	return Popen(split(command), stdout=PIPE, stderr=PIPE, **KWArgs)

def getProcessData(p):
	r"""Gets the process data from the given process. Could be used with launch, to get the processes'es output.
	"""
	out, err = [x.decode('utf-8') for x in p.communicate()]

	return {'out': out, 'err': err, 'code': p.returncode}

class CwdSwitch:
	r"""Helps with switching the CWD.
	"""
	def __init__(self, baseDir=None):
		self.cwd = baseDir or os.getcwd()

	def switch(self, newCwd):
		os.chdir(newCwd)

	def restore(self):
		os.chdir(self.cwd)

def assertShell(ShellResult, errorLine=None):
	r"""Asserts the success of a shell command.
	"""
	dump(getPretty(ShellResult))

	if ShellResult['code']:
		errorStr = ShellResult['err'] or ShellResult['out']

		if errorLine is not None:
			errorStr = getNthLine(errorStr, errorLine)

		raise Exception(errorStr)

	return ShellResult['out']

def extendEnv(**Extensions):
	Env = dict(**os.environ)
	Env.update(Extensions)

	return Env

# Init
def init():
	global split

	if 'win' not in sys.platform:
		split = shlexSplit

	else:
		from re import compile
		quoted = compile('^"(.+)"$')

		def _split(command): # #Note: This is just a crude implementaion to split windows command lines.
			Parts = []

			for item in shlexSplit(command, posix=False):
				match = quoted.match(item)

				Parts.append(match.groups(1)[0] if match else item)

			return Parts

		split = _split

init()



r"""
A module to help with development.
"""
import json

from laufire.dev import getPretty
from laufire.filesys import basename, copy, ensureParent, joinPaths
from laufire.flow import rob
from laufire.logger import debug
from laufire.shell import call, debugCall, assertShell

# Helpers
def getTgtName(tgtName, srcPath):
	return tgtName if tgtName else basename(srcPath)

class SSHClientMocker:
	def __init__(self, Config):
		Paths = Config['Gateway']['Paths']
		self.mockBase = Paths['base']

	def execute(self, command, **KWArgs):
		return call(command, **KWArgs)

	def upload(self, srcPath, tgtPath): # #Note: Uploads are done always to the temp dir.
		debug('uploading %s -> %s' % (srcPath, tgtPath))
		ensureParent(tgtPath)
		return copy(srcPath, tgtPath)

# Exports
class SSHBridgeMocker:
	r"""A class to help with the development of gateway scripts, by executing the commands locally.

	# #Note: This class doesn't fully mock SSHBridge as the need didn't arise.
	"""
	def __init__(self, Config):
		self.Config = Config
		self.Client = SSHClientMocker(Config)

		Config['Gateway']['Python']['binary'] = 'python' # #Pending: This is a quick fix. Remove it, once the remote can locate its python through shell.

		self.mockScriptTpl = '%s %s/%%s' % (Config['Gateway']['Python']['binary'], Config['Gateway']['Paths']['private'])

	def __getattr__(self, attr):
		r"""
		Allows the access of the methods of the Client.
		"""
		return getattr(self.Client, attr)

	def iexecute(self, command, **KWArgs):
		r"""Interpolates the command with the Config, before executing.
		"""
		return self.Client.execute(command.format(**self.Config['Gateway']), **KWArgs)

	def callScript(self, ecCommand):
		out = assertShell(call(self.mockScriptTpl % ecCommand, shell=True))

		if out:
			Out =	rob(lambda: json.loads(out))

			if Out is None:
				raise Exception(out)

			debug(getPretty(Out))
			return Out

	def debugScript(self, ecCommand):
		r"""Helps with debugging the gateway scripts.
		"""
		debugCall(self.mockScriptTpl % ecCommand)

	def upload(self, srcPath, tgtPath=''):
		if not tgtPath:
			tgtPath = joinPaths(self.Config['Gateway']['Paths']['temp'], getTgtName(None, srcPath))

		else:
			tgtPath = tgtPath.format(**self.Config['Gateway'])

		return self.Client.upload(srcPath, tgtPath)



r"""
A module to help with decorating functions.
"""
import json

from laufire.utils import getMD5ForIterable

# State
Memory = {}

# Helpers
def getHash(Data):
	r"""Returns the hash of the JSON of the given Data.
	"""
	return getMD5ForIterable(getJSON(Data, sort_keys=True)) # #Note: JSON output has to be sorted by keys, for the output to be deterministic.

def getJSON(value, sort_keys=True):
	return json.dumps(value, ensure_ascii=False, sort_keys=sort_keys)

def _getMemoizeKey(function, *Args, **KWArgs):
	Data = {

		'Args': [id(arg) for arg in Args],
		'KWArgs': {k: id(v) for k, v in KWArgs.iteritems()}
	}

	return '%s.%s/%s' % (function.__module__, function.__name__, getHash(Data))

# Exports
def wrap(function, wrapper):
	r"""
	Wraps the function with the given wraper.
	"""
	# Copy the function signature.
	wrapper.__module__ = function.__module__
	wrapper.__name__ = function.__name__
	wrapper.__doc__ = function.__doc__

# Decorators
def memoize(function):
	r"""
	A decorator to help with buffering (non-persistent) the results of function calls.
	# #Note: Use memoize, only when the calls are sufficiently costly. Else, memoization itself could add to the cost.
	"""
	def wrapper(*Args, **KWArgs):

		key = _getMemoizeKey(function, *Args, **KWArgs)

		if key in Memory:
			return Memory[key]

		Ret = function(*Args, **KWArgs)

		Memory[key] = Ret

		return Ret

	wrap(function, wrapper)

	return wrapper

# Tools
def rerun(function, *Args, **KWArgs):
	r"""
	Re-runs the memoized function and re-memoizes the result.
	"""
	key = _getMemoizeKey(function, *Args, **KWArgs)

	if key in Memory:
		del Memory[key] # Clear existing data, thus force a recalculation.

	return function(*Args, **KWArgs) # #Note: This value will be memoized.



#!/usr/bin/env python

from os import path
from setuptools import setup, find_packages

# Helpers
def getRequirementsFromFile(filePath):
  return [l for l in open(filePath, 'rb').read().splitlines() if l and l[0] != '#']

def getRequirements():
  scriptDir = path.abspath(path.dirname(__file__))
  reqFile = 'requirements.txt'
  
  return getRequirementsFromFile(path.join(scriptDir, reqFile))

# Data
description = 'laufire - a set of utilities to help with automation'
srcDir = 'dev'

setup(
  name='laufire',
  version='0.0.3',
	package_dir={'': srcDir},
  packages=find_packages(srcDir),
  description=description,
  long_description=description,
	# url='' #ToDo: Add a URL, probablly to the corporate page or to the docs.
  author='Laufire Technologies',
  platforms='any',
  include_package_data=True,
  install_requires=getRequirements(),
	python_requires='>=2.7.10', #ToDo: Find and add the minimum required verion of python. This requires a scanning of the requirements and/or a test suite to figure out the answer.
	# dependency_links = [''], #Note: This could be helpful, in hosting a minimal package repositoty.
  license="MIT",
  zip_safe=True,
  keywords='laufire utilities automation MIT',
  classifiers=[
    'Development Status :: 2 - Pre-Alpha',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'Operating System :: OS Independent',
    'Natural Language :: English',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Topic :: Software Development',
    'Topic :: Utilities',
  ],
  # test_suite='', # #Note: Tests are run from ops/main.py, for simplicity.
)



r"""
debug
-----

Calls to help with debugging scripts.
"""

import sys
import inspect

def obj2dict(obj):
	return dict((name, getattr(obj, name)) for name in dir(obj) if not name.startswith('_'))

def getCallerInfo(level=2, formatted=True):
	r"""Used to get the caller of the current function.

	Args:
		level: The *nth* level of the stack.
	"""
	stack = inspect.stack()

	if level > len(stack):
		return

	Info = inspect.getframeinfo(stack[level][0])

	if not formatted:
		return Info

	ret = ''

	for item in ['filename', 'function', 'lineno']:
		ret += '%s\t%s\n' % (item, getattr(Info, item, ''))

	ret += 'code\t%s\n' % Info.code_context[0]

	return ret

def err(string):
	r"""Logs the given string to the stderr, to help with debugging.
	"""
	sys.stderr.write('%s\n' % string)
